/** 
 * @file Initialise et configure le serveur Express sur lequel tourne le back.
 * 
 * La configuration inclut tout le _middleware_ définissant les API et les services
 * nécessaire utilisés, comme `express-session`, GraphiQL, GraphQL Voyager.
 * @author manifold
*/
import express from 'express';
import cookieParser from 'cookie-parser';
import { express as graphqlVoyager } from 'graphql-voyager/middleware';
import graphqlHTTP from 'express-graphql'; // new name of 'graphql-server-express'. cf npmjs.com
import { ensureLoggedIn } from 'connect-ensure-login';
import flash from 'connect-flash'; // utilise pour admin_view
import passport from 'passport';
import session from 'express-session';
import bodyParser from 'body-parser';
import favicon from 'serve-favicon';
import morgan from 'morgan';
import path from 'path';
import fs from 'fs';
import cors from 'cors';

import schema from './graphql/schema';
import './auth';

const app = express(); 
//The app object conventionally denotes the Express application 

// Parse incoming HTTP request bodies, available under the req.body property
// cf www.npmjs.com/package/body-parser
app.use(bodyParser.json()); //parses bodies of media type "application/json"
app.use(bodyParser.urlencoded({ //parses bodies of media type "application/x-www-form-urlencoded"
    extended: true //use qs library (quoi que ca veuille dire o.O)
}));
app.use(cookieParser());

const configPath = path.resolve('./', 'ldap_config.json');
const config = JSON.parse(fs.readFileSync(configPath, 'utf8'));

// Config de passport : le "import './auth';" plus haut execute la configuration de l'objet passport. cf, donc, auth.js

// Définit les paramètres de stockage des sessions.
app.use(session({
    secret: config.sessionSecret,
    resave: true,
    saveUninitialized: false
}));
app.use(passport.initialize());
app.use(passport.session());


// cache le fait que l'application tourne sous Express dans le header HTTP.
app.disable('x-powered-by');

// setting up view engine for pug
console.log("Running at",__dirname);
let viewpath = path.resolve(__dirname,'views');
app.set('views', viewpath);
app.set('view engine', 'pug');

// favicon: capital sigma symbol
app.use(favicon(path.resolve('./','assets','favicon.ico')));

// specifies path to static assets
app.use('/assets',express.static(path.resolve('./','assets')));

// Morgan is middleware for logging requests
app.use(morgan('dev'));

const defaultUser = require('./../ldap_connexion_config.json');

// Options de configuration pour le _middleware_ `cors`.
// CORS = Cross Origin Resource Sharing
const corsOptions = {
    origin: 'http://localhost:8888', // Configures the Access-Control-Allow-Origin CORS header. i.e. specifies that sigma-back wants to make resources accessible to this site (and this site only)
    credentials: true // Configures the Access-Control-Allow-Credentials CORS header. i.e. allows cookies to be included on cross-origin requests
};
app.use(cors(corsOptions));

import jwt from 'jsonwebtoken';

const SECRET_KEY = "azojgc;aegpfrihzcksdlmpqsqkx";

/*
const addUser = async (req, res, next) => {
    const token = req.cookies.csrftoken;
    if (!token) return next();
    console.log(`Token is ${token}`);

    try {
        const { user } = jwt.verify(token, SECRET_KEY);
        req.user = user;
    } catch (err) {
        console.log('Cookie error',err);
    }
};

app.use(addUser);
*/




//endpoint for frontend's authentication requests

//with custom callback:
//http://www.passportjs.org/docs/authenticate/#custom-callback
// http://toon.io/understanding-passportjs-authentication-flow/

app.post('/login', (req, res, next) => {
    passport.authenticate('ldapauth', (err, user, info) => {
        // If an exception occurred
        if (err) {
            console.log(err);
            return res.status(err.status).json({
                message: "Exception raised in backend process during authentication: " + err,
                authSucceeded: false
            });
            // return next(err); // handle error? or drop request and answer with res.json()?
        }
        // If authentication failed, user will be set to false
        if (!user) {
            return res.status(401).json({ 
                message: "Authentication failed: " + info.message,
                authSucceeded: false
            });
        }
        
        req.login(user, (err) => {
            // If an exception occurred at login
            if (err) { 
                console.log(err);
                return res.status(err.status).json({
                    message: "Exception raised in backend process during login: " + err,
                    authSucceeded: false
                });
                // return next(err); // handle error? or drop request and answer with res.json()?
            }
            // If all went well
            res.json({
                message: 'Authentication succeeded',
                authSucceeded: true
            });
        });
    })(req, res, next);
});

//without custom callback:
/*
// http://toon.io/understanding-passportjs-authentication-flow/
app.post('/login',
    passport.authenticate('ldapauth'),
    function (req, res) {
        // If this function gets called, authentication was successful.
        // `req.user` contains the authenticated user.
        console.log("Frontend authentication succeeded");
        res.json({
            message: 'Authentication succeeded',
            authSucceeded: true
        }); 
    }
);
*/


app.use('/graphql', 
    bodyParser.json(), // parse incoming HTTP request (req) as a JSON
    graphqlHTTP(async (req, res, params) => {
        // vary the options *on a per-request basis*
        let uid;
        let password;

        console.log("Responding to graphql request...");
        console.log(`User ${req.user ? req.user.uid : "none"}`);
        console.log("User is authenticated:",req.isAuthenticated());
        
        if(req.isAuthenticated()) {
            try {
                uid = req.user.uid;
                password = "mythe";
            } catch (err) {
                console.log("Error: req is authenticated, but pb when trying to extract uid from req.user. Probably user was either not serialized or not deserialized properly");
                console.log(err);
            }
        } else {
            // FOR DEVELOPMENT ONLY. for production, replace with a "publicUser" or "notLoggedInUser" or something.
            uid = defaultUser.dn.split("=")[1].split(",")[0];
            password = defaultUser.passwd;
        }

        console.log("Cookies:",req.cookies);

        return {
            schema,
            graphiql: true, // gives access to graphiql if request is detected to be from browser (je crois)
            context: { user: { uid: uid, password: password } } // accessible in every single resolver as the third argument
        };
    })
);

// GraphQL voyager affiche une représentation sous forme de graphe du schema GraphQL
app.use('/voyager',
    /*ensureLoggedIn('/login'),*/
    graphqlVoyager({ endpointUrl: '/graphql' })
);

// connect-flash is middleware for flashing messages
// used in sigma-back's admin interface
app.use(flash());

export default app;
